import java.awt.Image;
import javax.swing.ImageIcon;
import java.awt.event.KeyEvent;
import java.awt.Rectangle;
import java.util.HashMap;
import java.util.Map;
import java.io.*;

public class Hero implements Serializable
{

    //stringy cest k obrazkum    
    private String hero_crouch_static_left = "hero_crouch_static_left.gif";
    private String hero_crouch_static_right = "hero_crouch_static_right.gif";
    private String hero_crouch_to_stand_left = "hero_crouch_to_stand_left.gif";
    private String hero_crouch_to_stand_right = "hero_crouch_to_stand_right.gif";
    private String hero_crouch_walk_left = "hero_crouch_walk_left.gif";
    private String hero_crouch_walk_right = "hero_crouch_walk_right.gif";
    private String hero_stand_static_left = "hero_stand_static_left.gif";
    private String hero_stand_static_right = "hero_stand_static_right.gif";
    private String hero_stand_to_crouch_left = "hero_stand_to_crouch_left.gif";
    private String hero_stand_to_crouch_right = "hero_stand_to_crouch_right.gif";
    private String hero_walking_left = "hero_walking_left.gif";
    private String hero_walking_right = "hero_walking_right.gif";

    //to co kreslim na platno
    private Image image;

    private int id = -1;
    //samotna pozice na platne
    public double x;
    public double y;

    //sirka spritu
    private int width;
    private int height;
    private boolean visible;

    /*
     * zmeny provedene stisky klaves
     * problemy pri drzeni obou sipek zaroven,
     * jedna vyrusuje druou pri release checkX = 0
     * Je nutno pouzit dve promenne
     */
    private int checkXNegative;
    private int checkXPositive;
    private int checkY;

    //fyzika skoku a padu
    private double jumpHeight = 100;
    private double jumpSpeed = 2.5;
    private double jumpAcc = 0.03;
    private double jumpAccTmp = jumpSpeed;
    private double fallSpeed = 2.5;
    private double fallTmp = 0;
    private double fallAcc = 0.04;
    private boolean jump; 
    private boolean isOnGround;

    protected ImageIcon ii;

    //orientace pandulaka
    private boolean isStanding;
    private boolean isLeft;
    private boolean isRunning;
    protected int health = 100; 

    // mapovani klaves, kvuli multitouch, vice dole
    Map<Integer, Integer> keyMap = new HashMap<Integer, Integer>();

    private HeroStats stats;
    private NetworkClient nc;

    public boolean canShoot = false;

    public Hero()
    {
        x = 10;
        y = 10;    
        ImageIcon ii = new ImageIcon(this.getClass().getResource(hero_stand_static_right));
        image = ii.getImage();
        width = image.getWidth(null);
        height = image.getHeight(null);
        visible = true;
        isStanding = true;
        isOnGround = false;
        isLeft = false;
        isRunning = false;
        stats = new HeroStats();
        stats.width = ii.getIconWidth();
        stats.height = ii.getIconHeight();
    }

    /**
     * Snazime se o neco realistejsi a plynulejsi pohyb pohyb:
     * Pri padu jedem od -> 0 - fallSpeed px/cas
     * Pri skoku jedem od -> jumpSpeed - 0 px/cas 
     * 
     * Pri dosazeni maxspeed uz hodnoty neprelezem
     * Na bazi zrychleni, viz. implicitni hodnoty nahore
     */

    public void action()
    {       
        //skok
        if(jump)
        {
            jumpAccTmp -= jumpAcc;
            if(jumpAccTmp <= 0.0)
            {
                jump = false;
                jumpAccTmp = jumpSpeed;
            }
            y -= jumpAccTmp;
        }

        //volny pad
        if((!isOnGround) && (!jump))
        {
            if(fallTmp <= fallSpeed)
                fallTmp += fallAcc;
            y += fallTmp;
        }
        else
            fallTmp = 0;

        //pridavani zmenenych hodnot    
        x += checkXPositive;
        x += checkXNegative;
        //System.out.println("REKNI MI KOLIK JE POSITIVE X - "+checkXPositive);
        stats.fillStats(id,health,x,y,checkXPositive,checkXNegative,jump,isOnGround,isStanding,isLeft, isRunning);
    }

    public int getX() 
    {
        return (int)(x);
    }

    public int getY() 
    {
        return (int)(y);
    }

    public Image getImage() {
        return image;
    }

    public Image getImage(HeroStats player) 
    {
        if(player.isStanding)
        {
            if(player.isLeft)
            {
                if(player.isRunning)
                    ii = new ImageIcon(this.getClass().getResource(hero_walking_left));
                else
                    ii = new ImageIcon(this.getClass().getResource(hero_stand_static_left));      
            }
            else
            {
                if(player.isRunning)
                    ii = new ImageIcon(this.getClass().getResource(hero_walking_right));
                else
                    ii = new ImageIcon(this.getClass().getResource(hero_stand_static_right));      
            }
        }
        else
        {
            if(player.isLeft)
            {
                if(player.isRunning)
                    ii = new ImageIcon(this.getClass().getResource(hero_crouch_walk_left));
                else
                    ii = new ImageIcon(this.getClass().getResource(hero_crouch_static_left));      
            }
            else
            {
                if(player.isRunning)
                    ii = new ImageIcon(this.getClass().getResource(hero_crouch_walk_right));
                else
                    ii = new ImageIcon(this.getClass().getResource(hero_crouch_static_right));      
            }
        }
        return ii.getImage();
    }

    public void setVisible(boolean visible) {
        this.visible = visible;
    }

    public void setIsOnGround(boolean statement) {
        isOnGround = statement;
    }

    public void setJump(boolean statement) {
        jump = statement;
    }
    
    public void setHealth(int health) 
    {
        this.health = health;
    }
    
    public boolean isAlive()
    {
        if(health <= 0)
            return false;
        return true;
    }

    public boolean isVisible() {
        return visible;
    }

    public HeroStats getStats() {
        return stats;
    }

    public void setJumpDefaults()
    {
        jumpHeight = 100;
        jumpSpeed = 2.5;
        jumpAcc = 0.03;
        jumpAccTmp = jumpSpeed;
    }

    public void setPosition(int x, int y) {
        this.x = x;
        this.y = y;
        isStanding = false;
    }

    public Rectangle getBounds() {
        return new Rectangle((int)(x), (int)(y), width, height);
    }

    /**
     * Protoze potrebujeme pracovat s vice stisk. klavesama najednou,
     * musime udelat mapu klaves, ktere jsou stisknuty a v pripade,
     * ze dojde k releasu je odstranit z mapy. Kazdym cyklem se provadi
     * to co je v mape.
     */

    public void keyPressed(KeyEvent e, NetworkClient nc) 
    {
        this.nc = nc;
        int key = e.getKeyCode();
        //System.out.println(key);
        boolean isInKeyMap = keyMap.containsKey(key);
        if (!isInKeyMap)
        {
            keyMap.put(new Integer(e.getKeyCode()), new Integer(keyMap.size()));
            //sendStats(nc);
        }
    }

    public void keyReleased(KeyEvent e, NetworkClient nc) {
        int key = e.getKeyCode();
        //System.out.println(e);

        keyMap.remove(e.getKeyCode());
        ImageIcon ii;

        if (key == 37) {
            checkXNegative = 0;
            if(isStanding)
            {
                isLeft = true;
                isRunning = false;
                isStanding = true;
                ii = new ImageIcon(this.getClass().getResource(hero_stand_static_left));
            }
            else
            {
                isLeft = true;
                isRunning = false;
                isStanding = false;
                ii = new ImageIcon(this.getClass().getResource(hero_crouch_static_left));
            }
            image = ii.getImage();
            width = image.getWidth(null);
            height = image.getHeight(null);
            image.flush();
        }

        if (key == 39) {
            checkXPositive = 0;
            if(isStanding)
            {
                isLeft = false;
                isRunning = false;
                isStanding = true;
                ii = new ImageIcon(this.getClass().getResource(hero_stand_static_right));
            }
            else
            {
                isLeft = false;
                isRunning = false;
                isStanding = false;
                ii = new ImageIcon(this.getClass().getResource(hero_crouch_static_right));
            }
            image = ii.getImage();
            width = image.getWidth(null);
            height = image.getHeight(null);
            isLeft = false;
            image.flush();
        }

        if (key == 38) {

        }

        if (key == 40) {
            if(isLeft)
            {
                isLeft = true;
                isStanding = true;
                ii = new ImageIcon(this.getClass().getResource(hero_crouch_to_stand_left));
            }
            else
            {
                isLeft = false;
                isStanding = true;
                ii = new ImageIcon(this.getClass().getResource(hero_crouch_to_stand_right));
            }
            image = ii.getImage();
            width = image.getWidth(null);
            height = image.getHeight(null);
            image.flush();
        }

        stats.fillStats(id,health,x,y,checkXPositive,checkXNegative,jump,isOnGround,isStanding,isLeft, isRunning);
        //sendStats(nc);
    }

    /**
     * Toto proste bylo nutne udelat, nebot samotny keyadapter
     * ceka na stisk klavesy nez zacne neco tvorit.
     * Ja ale chci aby tvoril furt dokud je tlacitko v pameti.
     */

    public void doKeyAction() 
    {
        ImageIcon ii;
        if (keyMap.containsKey(37)) {
            if(isStanding)
            {
                isLeft = true;
                isRunning = true;
                isStanding = true;
                ii = new ImageIcon(this.getClass().getResource(hero_walking_left));

            }
            else
            {
                isLeft = true;
                isRunning = true;
                isStanding = false;
                ii = new ImageIcon(this.getClass().getResource(hero_crouch_walk_left));
            }

            image = ii.getImage();
            width = image.getWidth(null);
            height = image.getHeight(null);
            //*************************
            checkXNegative = -1;
        }

        if (keyMap.containsKey(39)) {
            if(isStanding)
            {
                isLeft = false;
                isRunning = true;
                isStanding = true;
                ii = new ImageIcon(this.getClass().getResource(hero_walking_right));
            }
            else
            {
                isLeft = false;
                isRunning = true;
                isStanding = false;
                ii = new ImageIcon(this.getClass().getResource(hero_crouch_walk_right));
            }
            image = ii.getImage();
            width = image.getWidth(null);
            height = image.getHeight(null);
            isLeft = false;
            //******************************
            checkXPositive = 1;
        }

        if (keyMap.containsKey(38)) {
            if(isOnGround)
            {
                isOnGround = false;
                if(!jump)
                    jump = true;
            }
        }
        if (keyMap.containsKey(40)) {
            if(isStanding)
            { 
                if(isLeft)
                {
                    isLeft = true;
                    isStanding = false;
                    ii = new ImageIcon(this.getClass().getResource(hero_stand_to_crouch_left));
                }
                else
                {
                    isLeft = false;
                    isStanding = false;
                    ii = new ImageIcon(this.getClass().getResource(hero_stand_to_crouch_right));
                }
                image = ii.getImage();
                width = image.getWidth(null);
                height = image.getHeight(null);
                image.flush();
            }
        }

        if (keyMap.containsKey(32)) //mezernik, strela
        {
            if(canShoot)
            {
                canShoot = false;
                nc.makeShot(stats);
            }
        }

    }

    public void sendStats(NetworkClient nc)
    {
        try
        {
            nc.send(stats);
        }
        catch(Exception e)
        {
            System.out.println(e);
        }
    }
}
